"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sharp_1 = __importDefault(require("sharp"));
const internal_1 = require("../../internal");
async function mergeLayers(document, name, inputLayers, copy) {
    let layers;
    if (inputLayers) {
        const layerObjects = inputLayers.map((l) => ((typeof l === "string") || (typeof l === "number")) ? document.getLayer(l) : l);
        const invalidLayerIndex = layerObjects.findIndex((l) => l && l.document !== document);
        if (invalidLayerIndex !== -1)
            throw new Error(`Layer at index ${invalidLayerIndex} isn't part of this document`);
        const unknownLayerIndex = layerObjects.findIndex((l) => !l);
        if (unknownLayerIndex !== -1) {
            const inputLayer = inputLayers[unknownLayerIndex];
            throw new Error(`Unknown layer with ${typeof inputLayer === "string" ? `name '${inputLayer}'` : `index ${inputLayer}`}`);
        }
        layers = layerObjects;
    }
    else {
        layers = [...document.layers];
    }
    const position = Math.max(...layers.map((l) => l.position)) + 1;
    document._debug(`Merging ${layers.length} layers (${layers.map((l) => l.name).join(", ")})${copy ? " via copy" : ""} into '${name}' at position ${position}`);
    const exportedLayersPromises = layers.map((l) => l.exportTo("png", "buffer", true));
    let exportedLayers = await Promise.all(exportedLayersPromises);
    const canvas = sharp_1.default({
        create: {
            width: document.width,
            height: document.height,
            channels: 4,
            background: { r: 0, g: 0, b: 0, alpha: 0 }
        }
    });
    const croppedLayersPromises = exportedLayers.map((l, index) => {
        if (layers[index].left >= 0 &&
            layers[index].top >= 0 &&
            layers[index].left + l.width <= document.width &&
            layers[index].top + l.height <= document.height)
            return l.data;
        const cropLeft = layers[index].left < 0 ? (layers[index].left / -1) : 0;
        const cropTop = layers[index].top < 0 ? (layers[index].top / -1) : 0;
        const cropWidth = document.width - layers[index].left;
        const cropHeight = document.height - layers[index].top;
        if (((cropWidth < l.width ? cropWidth : l.width) - cropLeft <= 0) || ((cropHeight < l.height ? cropHeight : l.height) - cropTop <= 0))
            return;
        return sharp_1.default(l.data)
            .extract({
            left: cropLeft,
            top: cropTop,
            width: (cropWidth < l.width ? cropWidth : l.width) - cropLeft,
            height: (cropHeight < l.height ? cropHeight : l.height) - cropTop
        })
            .toFormat("png")
            .toBuffer();
    });
    const croppedLayersPromisesFiltered = croppedLayersPromises.filter((l) => l);
    const croppedLayers = await Promise.all(croppedLayersPromisesFiltered);
    canvas.composite(croppedLayers.map((l, index) => {
        const blendMode = layers[index].blendMode;
        let thisBlendMode = "";
        if (blendMode === "normal")
            thisBlendMode = "over";
        else if (blendMode === "darken")
            thisBlendMode = "darken";
        else if (blendMode === "multiply")
            thisBlendMode = "multiply";
        else if (blendMode === "colorBurn")
            thisBlendMode = "color-burn";
        else if (blendMode === "lighten")
            thisBlendMode = "lighten";
        else if (blendMode === "screen")
            thisBlendMode = "screen";
        else if (blendMode === "colorDodge")
            thisBlendMode = "color-dodge";
        else if (blendMode === "linearDodge")
            thisBlendMode = "add";
        else if (blendMode === "overlay")
            thisBlendMode = "overlay";
        else if (blendMode === "softLight")
            thisBlendMode = "soft-light";
        else if (blendMode === "hardLight")
            thisBlendMode = "hard-light";
        else if (blendMode === "difference")
            thisBlendMode = "difference";
        else if (blendMode === "exclusion")
            thisBlendMode = "exclusion";
        return {
            input: l,
            left: layers[index].left < 0 ? 0 : layers[index].left,
            top: layers[index].top < 0 ? 0 : layers[index].top,
            blend: thisBlendMode
        };
    }));
    const mergedLayer = await canvas.toFormat("png").toBuffer();
    const newLayer = new internal_1.Layer({
        name,
        buffer: mergedLayer,
        position
    }, document);
    await newLayer._initialize;
    if (!copy)
        layers.forEach((l) => l.remove());
    return newLayer;
}
exports.default = mergeLayers;
